@@ -66,12 +66,22 @@ use core::convert::TryInto;
 use crate::flash::ACR;
 use crate::time::rate::*;
 
-/// Extension trait that constrains the `RCC` peripheral
-pub trait RccExt {
-    /// Constrains the `RCC` peripheral so it plays nicely with the other abstractions
+impl crate::private::Sealed for RCC {}
+
+/// Extension trait that constrains the []`RCC`] peripheral
+pub trait RccExt: crate::private::Sealed {
+    /// Constrains the [`RCC`] peripheral.
+    ///
+    /// Consumes the [`pac::RCC`] peripheral and converts it to a [`HAL`] internal type
+    /// constraining it's public access surface to fit the design of the `HAL`.
+    ///
+    /// [`pac::RCC`]: `crate::pac::RCC`
+    /// [`HAL`]: `crate`
     fn constrain(self) -> Rcc;
 }
 
+mod enable;
+
 impl RccExt for RCC {
     fn constrain(self) -> Rcc {
         Rcc {
@@ -86,9 +96,8 @@ impl RccExt for RCC {
 
 /// Constrained RCC peripheral
 ///
-/// An instance of this struct is acquired by calling the
-/// [`constrain`](RccExt::constrain) function on the
-/// [`RCC`](crate::pac::RCC) struct.
+/// An instance of this struct is acquired by calling the [`constrain`](RccExt::constrain) function
+/// on the [`RCC`](crate::pac::RCC) struct.
 ///
 /// ```
 /// let dp = pac::Peripherals::take().unwrap();
@@ -120,18 +129,6 @@ pub struct AHB {
     _0: (),
 }
 
-impl AHB {
-    pub(crate) fn enr(&mut self) -> &rcc::AHBENR {
-        // NOTE(unsafe) this proxy grants exclusive access to this register
-        unsafe { &(*RCC::ptr()).ahbenr }
-    }
-
-    pub(crate) fn rstr(&mut self) -> &rcc::AHBRSTR {
-        // NOTE(unsafe) this proxy grants exclusive access to this register
-        unsafe { &(*RCC::ptr()).ahbrstr }
-    }
-}
-
 /// Advanced Peripheral Bus 1 (APB1) registers
 ///
 /// An instance of this struct is acquired from the [`RCC`](crate::pac::RCC) struct.
@@ -145,18 +142,6 @@ pub struct APB1 {
     _0: (),
 }
 
-impl APB1 {
-    pub(crate) fn enr(&mut self) -> &rcc::APB1ENR {
-        // NOTE(unsafe) this proxy grants exclusive access to this register
-        unsafe { &(*RCC::ptr()).apb1enr }
-    }
-
-    pub(crate) fn rstr(&mut self) -> &rcc::APB1RSTR {
-        // NOTE(unsafe) this proxy grants exclusive access to this register
-        unsafe { &(*RCC::ptr()).apb1rstr }
-    }
-}
-
 /// Advanced Peripheral Bus 2 (APB2) registers
 ///
 /// An instance of this struct is acquired from the [`RCC`](crate::pac::RCC) struct.
@@ -170,15 +155,142 @@ pub struct APB2 {
     _0: (),
 }
 
-impl APB2 {
-    pub(crate) fn enr(&mut self) -> &rcc::APB2ENR {
-        // NOTE(unsafe) this proxy grants exclusive access to this register
-        unsafe { &(*RCC::ptr()).apb2enr }
-    }
+macro_rules! bus_struct {
+    ($($busX:ident => ($EN:ident, $en:ident, $RST:ident, $rst:ident),)+) => {
+        $(
+            impl $busX {
+                fn new() -> Self {
+                    Self { _0: () }
+                }
 
-    pub(crate) fn rstr(&mut self) -> &rcc::APB2RSTR {
-        // NOTE(unsafe) this proxy grants exclusive access to this register
-        unsafe { &(*RCC::ptr()).apb2rstr }
+                #[allow(unused)]
+                fn enr(&self) -> &rcc::$EN {
+                    // NOTE(unsafe) this proxy grants exclusive access to this register
+                    unsafe { &(*RCC::ptr()).$en }
+                }
+
+                #[allow(unused)]
+                fn rstr(&self) -> &rcc::$RST {
+                    // NOTE(unsafe) this proxy grants exclusive access to this register
+                    unsafe { &(*RCC::ptr()).$rst }
+                }
+            }
+        )+
+    };
+}
+
+bus_struct! {
+    AHB => (AHBENR, ahbenr, AHBRSTR, ahbrstr),
+    APB1 => (APB1ENR, apb1enr, APB1RSTR, apb1rstr),
+    APB2 => (APB2ENR, apb2enr, APB2RSTR, apb2rstr),
+}
+
+/// Bus associated to peripheral
+pub trait RccBus: crate::Sealed {
+    /// The underlying bus peripheral
+    type Bus;
+}
+
+/// Enable/disable peripheral
+pub trait Enable: RccBus {
+    /// Enables peripheral
+    fn enable(bus: &mut Self::Bus);
+
+    /// Disables peripheral
+    fn disable(bus: &mut Self::Bus);
+
+    /// Check if peripheral enabled
+    fn is_enabled() -> bool;
+
+    /// Check if peripheral disabled
+    fn is_disabled() -> bool;
+
+    /// Enables peripheral
+    ///
+    /// # Safety
+    ///
+    /// Takes access to RCC internally, so you have to make sure
+    /// you don't have race condition accessing RCC registers
+    unsafe fn enable_unchecked();
+
+    /// Disables peripheral
+    ///
+    /// # Safety
+    ///
+    /// Takes access to RCC internally, so you have to make sure
+    /// you don't have race condition accessing RCC registers
+    unsafe fn disable_unchecked();
+}
+
+/// Reset peripheral
+pub trait Reset: RccBus {
+    /// Resets peripheral
+    fn reset(bus: &mut Self::Bus);
+
+    /// # Safety
+    ///
+    /// Resets peripheral. Takes access to RCC internally
+    unsafe fn reset_unchecked();
+}
+
+/// Frequency on bus that peripheral is connected in
+pub trait BusClock {
+    /// Calculates frequency depending on `Clock` state
+    fn clock(clocks: &Clocks) -> Hertz;
+}
+
+impl<T> BusClock for T
+where
+    T: RccBus,
+    T::Bus: BusClock,
+{
+    fn clock(clocks: &Clocks) -> Hertz {
+        T::Bus::clock(clocks)
+    }
+}
+
+impl BusClock for AHB {
+    fn clock(clocks: &Clocks) -> Hertz {
+        clocks.hclk
+    }
+}
+impl BusClock for APB1 {
+    fn clock(clocks: &Clocks) -> Hertz {
+        clocks.pclk1
+    }
+}
+impl BusClock for APB2 {
+    fn clock(clocks: &Clocks) -> Hertz {
+        clocks.pclk2
+    }
+}
+
+/// Frequency on bus that timer is connected in
+pub trait BusTimerClock {
+    /// Calculates base frequency of timer depending on `Clock` state
+    fn timer_clock(clocks: &Clocks) -> Hertz;
+}
+
+impl<T> BusTimerClock for T
+where
+    T: RccBus,
+    T::Bus: BusTimerClock,
+{
+    fn timer_clock(clocks: &Clocks) -> Hertz {
+        T::Bus::timer_clock(clocks)
+    }
+}
+
+impl BusTimerClock for APB1 {
+    fn timer_clock(clocks: &Clocks) -> Hertz {
+        let pclk_mul = if clocks.ppre1 > 1 { 2 } else { 1 };
+        Hertz(clocks.pclk1.0 * pclk_mul)
+    }
+}
+impl BusTimerClock for APB2 {
+    fn timer_clock(clocks: &Clocks) -> Hertz {
+        let pclk_mul = if clocks.ppre2 > 1 { 2 } else { 1 };
+        Hertz(clocks.pclk2.0 * pclk_mul)
     }
 }
 
@@ -246,6 +358,7 @@ pub struct BDCR {
 }
 
 impl BDCR {
+    #[allow(unused)]
     pub(crate) fn bdcr(&mut self) -> &rcc::BDCR {
         // NOTE(unsafe) this proxy grants exclusive access to this register
         unsafe { &(*RCC::ptr()).bdcr }
@@ -296,6 +409,8 @@ pub(crate) struct PllConfig {
 /// Determine the [greatest common divisor](https://en.wikipedia.org/wiki/Greatest_common_divisor)
 ///
 /// This function is based on the [Euclidean algorithm](https://en.wikipedia.org/wiki/Euclidean_algorithm).
+// TODO(Sh3Rm4n): As num-traits is a indirecty dependecy of this crate through embedded-time,
+// use its implementation instead.
 fn gcd(mut a: u32, mut b: u32) -> u32 {
     while b != 0 {
         let r = a % b;
